{ nixpkgs ? import <nixpkgs> {}, compiler ? "default", doBenchmark ? false }:

let

  inherit (nixpkgs) pkgs;

  nanovgNeko = with haskellPackages; callPackage (
    { mkDerivation, base, bytestring, c2hs, containers, glew
    , hspec, inline-c, libGL, libGLU, QuickCheck, stdenv, text, vector
    }:
    mkDerivation {
      pname = "nanovg";
      version = "0.6.0.0";
      sha256 = "e35a2772f2a2e71916013cab70eeb8eedd6f66b5490ddf7ce237c8c76895d496";
      isLibrary = true;
      isExecutable = true;
      doCheck = false;
      libraryHaskellDepends = [ base bytestring containers text vector ];
      librarySystemDepends = [ glew libGL libGLU ];
      libraryPkgconfigDepends = [ glew ];
      libraryToolDepends = [ c2hs ];
      testHaskellDepends = [ base containers hspec inline-c QuickCheck ];
      homepage = "https://github.com/cocreature/nanovg-hs";
      description = "Haskell bindings for nanovg";
      license = stdenv.lib.licenses.isc;
    }) {};

  f = { mkDerivation, base, bytestring, clock, containers, deepseq
      , glib, linear, matrix, monad-loops, monad-parallel, mtl
      , OpenGL, random, sdl2, stdenv, stm, text, uuid, vector
      }:
      mkDerivation {
        pname = "affection";
        version = "0.0.0.9";
        src = ./.;
        configureFlags = [ "-fexamples" ];
        isLibrary = true;
        isExecutable = true;
        libraryHaskellDepends = [
          base bytestring clock containers glib linear monad-loops
          monad-parallel mtl OpenGL sdl2 stm text uuid vector
        ];
        executableHaskellDepends = [
          base containers deepseq linear matrix nanovgNeko OpenGL random sdl2 stm
        ];
        homepage = "https://github.com/nek0/affection#readme";
        description = "A simple Game Engine using SDL";
        license = stdenv.lib.licenses.lgpl3;
      };

  haskellPackages = if compiler == "default"
                       then pkgs.haskellPackages
                       else pkgs.haskell.packages.${compiler};

  variant = if doBenchmark then pkgs.haskell.lib.doBenchmark else pkgs.lib.id;

  drv = variant (haskellPackages.callPackage f {});

in

  if pkgs.lib.inNixShell then drv.env else drv
